<?php

namespace Drupal\views_sql_twig_relationship\Plugin\views\relationship;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\relationship\Standard;
use Drupal\views_sql_twig_fields\TwigSqlFields;
use Drupal\views\Views;

/**
 * SQL Twig relationship handler .
 *
 * @ingroup views_relationship_handlers
 *
 * @ViewsRelationship("views_sql_twig_relationship")
 */
class ViewsSqlTwigRelationship extends Standard {
  use TwigSqlFields;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $this->twigSqlDefineOptions($options);
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsform($form, $form_state);
    $this->twigSqlBuildRelationshipOptionsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    if ($this->options['twig_use'] == 'original') {
      return parent::query();
    }

    $this->ensureMyTable();
    $def = $this->definition;
    $this->regenerateSqlQuery();

    // Figure out what base table this relationship brings to the party.
    $table_data = Views::viewsData()->get($this->definition['base']);
    $base_field = empty($this->definition['base field']) ? $table_data['table']['base']['field'] : $this->definition['base field'];

    $base = $this->definition['base'] . '_' . $this->table;
    $def['table'] = $this->definition['base'];
    $def['field'] = $base_field;
    $def['left_table'] = $left = $this->tableAlias;
    $def['left_field'] = $left_field = $this->realField;

    $def['sql_condition'] = $this->options['sql'] . (
      ($this->options['twig_use'] == 'original') ?
      '' :
      " AND ($left.$left_field = $base.$base_field)"
      ) ;

    $def['adjusted'] = TRUE;

    if (!empty($this->options['required'])) {
      $def['type'] = 'INNER';
    }

    if (!empty($this->definition['extra'])) {
      $def['extra'] = $this->definition['extra'];
    }

    $join = Views::pluginManager('join')->createInstance('twig_sql_join', $def);

    // Use a short alias for this:
    $alias = $def['table'] . '_' . $this->table;

    $this->alias = $this->query->addRelationship($alias, $join, $this->definition['base'], $this->relationship);

    // Add access tags if the base table provide it.
    if (empty($this->query->options['disable_sql_rewrite']) && isset($table_data['table']['base']['access query tag'])) {
      $access_tag = $table_data['table']['base']['access query tag'];
      $this->query->addTag($access_tag);
    }
  }

}
